import os, json, shutil, urllib.request

opener = urllib.request.build_opener()
opener.addheaders = [("User-agent", "Mozilla/5.0")]
urllib.request.install_opener(opener)

DIR = os.path.abspath(os.getcwd())
cache_folder = os.path.join(DIR, "cache")
json_folder = os.path.join(cache_folder, "json")
if not os.path.isdir(json_folder):
    os.makedirs(json_folder, exist_ok=True)

ETAGFILE = "cache/json/etags.json"
etag_header = "If-None-Match"  # Header for checking if etag is updated
useragent = "Mozilla/5.0"


def download_file(url, file):
    # try:
    req = urllib.request.Request(url)
    with urllib.request.urlopen(req) as response, open(file, "wb+") as out_file:
        shutil.copyfileobj(response, out_file)
        print("file {} - Dowloaded".format(file))
    return file
    # except Exception as e:
    # 	return f"Failed to download file - {file} - {e}"


def accessETaggedFile(url, file):
    req = urllib.request.Request(url)
    req.add_header("User-Agent", useragent)
    etag = getEtag(file)
    if etag:
        req.add_header(etag_header, "{}".format(etag))
    try:
        with urllib.request.urlopen(req) as response, open(file, "wb+") as out_file:
            shutil.copyfileobj(response, out_file)
            headers = response.info()
            newetag = headers["ETag"]
            setEtag(file, newetag)
        print("file {} - Updated".format(file))
    except urllib.error.URLError as e:
        if (
            e.reason == "Not Modified"
        ):  # 304 error, what we want to see if nothing has been updated
            print("file {} - {}".format(file, e.reason))
        else:
            print("etag download error - {} - {}\n\n".format(file, e.reason))
            return None
    return file


def setEtag(tag, etag):
    if not os.path.isfile(ETAGFILE):
        print("No ETag file, initializing")
        with open(ETAGFILE, "w+", encoding="utf-8") as jfile:
            newfile = {"created_by": __file__}
            json.dump(
                newfile,
                jfile,
                indent=4,
            )
    newentry = {tag: etag}
    with open(ETAGFILE, "r", encoding="utf-8") as jfile:
        originaljfile = json.load(jfile)
    originaljfile.update(newentry)
    with open(ETAGFILE, "w", encoding="utf-8") as jfile:
        json.dump(
            originaljfile,
            jfile,
            indent=4,
        )


def getEtag(tag):
    try:
        with open(ETAGFILE, "r", encoding="utf-8") as json_file:
            jfile = json.load(json_file)
        try:
            etag = jfile[tag]
        except:
            etag = None
    except:
        etag = None
    return etag


# Super simple caching system, items all use a unique name and end up in /cache/json
# Uses etagging to reduce unnecessary load on the server
def getJson(distinctname, apiurl):
    try:
        jsonfile = os.path.join("cache/json", distinctname + ".json")
        jfile = accessETaggedFile(apiurl, jsonfile)
        return jfile
    except Exception as e:
        return print("{}\nfailed to download json file for {}".format(e, distinctname))


# Super simple caching system, items all use a unique name and end up in /cache/json
# Uses etagging to reduce unnecessary load on the server
def getCachedJson(distinctname):
    return os.path.join("cache/json", distinctname + ".json")
